@@ -91,6 +91,9 @@ function configure_mistral {
     # Configure action execution deletion policy
     iniset $MISTRAL_CONF_FILE api allow_action_execution_deletion True
 
+    # Don't use the default 0.0.0.0 it's good only for ipv4
+    iniset $MISTRAL_CONF_FILE api host $(ipv6_unquote $MISTRAL_SERVICE_HOST)
+
     if [ "$LOG_COLOR" == "True" ] && [ "$SYSLOG" == "False" ]; then
         setup_colorized_logging $MISTRAL_CONF_FILE DEFAULT tenant user
     fi
@@ -102,6 +105,12 @@ function configure_mistral {
     if [ "$MISTRAL_USE_MOD_WSGI" == "True" ]; then
         _config_mistral_apache_wsgi
     fi
+
+    if [[ ! -z "$MISTRAL_COORDINATION_URL" ]]; then
+        iniset $MISTRAL_CONF_FILE coordination backend_url "$MISTRAL_COORDINATION_URL"
+    elif is_service_enabled etcd3; then
+        iniset $MISTRAL_CONF_FILE coordination backend_url "etcd3+http://${SERVICE_HOST}:$ETCD_PORT"
+    fi
 }
 
 
@@ -118,7 +127,19 @@ function install_mistral {
     setup_develop $MISTRAL_DIR
 
     # installing python-nose.
-    real_install_package python-nose
+    if is_fedora; then
+        if python3_enabled; then
+            real_install_package python3-nose
+        else
+            real_install_package python2-nose
+        fi
+    elif is_ubuntu; then
+        if python3_enabled; then
+            real_install_package python3-nose
+        fi
+    else
+        real_install_package python-nose
+    fi
 
     if is_service_enabled horizon; then
         _install_mistraldashboard
@@ -152,6 +173,13 @@ function install_mistral_lib {
     fi
 }
 
+function install_mistral_extra {
+    if use_library_from_git "mistral-extra"; then
+        git_clone $MISTRAL_EXTRA_REPO $MISTRAL_EXTRA_DIR $MISTRAL_EXTRA_BRANCH
+        setup_develop $MISTRAL_EXTRA_DIR
+    fi
+}
+
 # start_mistral - Start running processes
 function start_mistral {
     # If the site is not enabled then we are in a grenade scenario
@@ -191,6 +219,11 @@ function stop_mistral {
     fi
 }
 
+function configure_tempest_for_mistral {
+    if is_service_enabled tempest; then
+        iniset $TEMPEST_CONFIG mistral_api service_api_supported True
+    fi
+}
 
 function cleanup_mistral {
     if is_service_enabled horizon; then
@@ -241,6 +274,7 @@ if is_service_enabled mistral; then
         echo_summary "Installing mistral"
         install_mistral
         install_mistral_lib
+        install_mistral_extra
         install_mistral_pythonclient
     elif [[ "$1" == "stack" && "$2" == "post-config" ]]; then
         echo_summary "Configuring mistral"
@@ -250,6 +284,9 @@ if is_service_enabled mistral; then
         echo_summary "Initializing mistral"
         init_mistral
         start_mistral
+    elif [[ "$1" == "stack" && "$2" == "test-config" ]]; then
+        echo_summary "Configuring Tempest for Mistral"
+        configure_tempest_for_mistral
     fi
 
     if [[ "$1" == "unstack" ]]; then
