@@ -1,5 +1,7 @@
 # Copyright 2013 - Mirantis, Inc.
 # Copyright 2018 - Extreme Networks, Inc.
+# Copyright 2019 - NetCracker Technology Corp.
+# Copyright 2020 Nokia Software.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License");
 #    you may not use this file except in compliance with the License.
@@ -27,6 +29,7 @@ SCOPE_TYPES = wtypes.Enum(str, 'private', 'public')
 
 class ScopedResource(object):
     """Utilities for scoped resources"""
+
     @classmethod
     def validate_scope(cls, scope):
         if scope not in SCOPE_TYPES.values:
@@ -58,7 +61,7 @@ class Workbook(resource.Resource, ScopedResource):
         return cls(id='123e4567-e89b-12d3-a456-426655440000',
                    name='book',
                    definition='HERE GOES'
-                        'WORKBOOK DEFINITION IN MISTRAL DSL v2',
+                              'WORKBOOK DEFINITION IN MISTRAL DSL v2',
                    tags=['large', 'expensive'],
                    scope='private',
                    project_id='a7eb669e9819420ea4bd1453e672c0a7',
@@ -89,9 +92,10 @@ class Workflow(resource.Resource, ScopedResource):
     name = wtypes.text
     namespace = wtypes.text
     input = wtypes.text
-
+    interface = types.jsontype
+    "input and output of the workflow"
     definition = wtypes.text
-    "Workflow definition in Mistral v2 DSL"
+    "workflow text written in Mistral v2 language"
     tags = [wtypes.text]
     scope = SCOPE_TYPES
     "'private' or 'public'"
@@ -106,16 +110,23 @@ class Workflow(resource.Resource, ScopedResource):
                    name='flow',
                    input='param1, param2',
                    definition='HERE GOES'
-                        'WORKFLOW DEFINITION IN MISTRAL DSL v2',
+                              'WORKFLOW DEFINITION IN MISTRAL DSL v2',
                    tags=['large', 'expensive'],
                    scope='private',
                    project_id='a7eb669e9819420ea4bd1453e672c0a7',
                    created_at='1970-01-01T00:00:00.000000',
                    updated_at='1970-01-01T00:00:00.000000',
-                   namespace='')
+                   namespace='',
+                   interface={"input": ["param1", {"param2": 2}],
+                              "output": []}
+                   )
 
-    @classmethod
-    def _set_input(cls, obj, wf_spec):
+    def set_attributes_from_spec(self, wf_spec):
+        #  Sets input and interface fields for the Workflow resource.
+        self._set_input(wf_spec)
+        self._set_interface(wf_spec)
+
+    def _set_input(self, wf_spec):
         input_list = []
 
         if wf_spec:
@@ -124,43 +135,169 @@ class Workflow(resource.Resource, ScopedResource):
             for param in input:
                 if isinstance(param, dict):
                     for k, v in param.items():
-                        input_list.append("%s=%s" % (k, v))
+                        input_list.append('%s="%s"' % (k, v))
                 else:
                     input_list.append(param)
 
-            setattr(obj, 'input', ", ".join(input_list) if input_list else '')
+            self.input = ", ".join(input_list) if input_list else ''
 
-        return obj
+    def _set_interface(self, wf_spec):
+        self.interface = {}
+
+        if wf_spec:
+            self.interface['input'] = wf_spec.get('input', [])
+            self.interface['output'] = [
+                output for output in wf_spec.get('output', {})
+            ]
 
     @classmethod
     def from_dict(cls, d):
         obj = super(Workflow, cls).from_dict(d)
 
-        return cls._set_input(obj, d.get('spec'))
+        obj.set_attributes_from_spec(d.get('spec'))
+
+        return obj
 
     @classmethod
-    def from_db_model(cls, db_model):
-        obj = super(Workflow, cls).from_db_model(db_model)
+    def from_db_model(cls, db_model, fields=()):
+        obj = super(Workflow, cls).from_db_model(db_model, fields=fields)
 
-        return cls._set_input(obj, db_model.spec)
+        obj.set_attributes_from_spec(db_model.get('spec'))
+
+        return obj
 
     @classmethod
     def from_tuples(cls, tuple_iterator):
         obj = cls()
         spec = None
+
         for col_name, col_val in tuple_iterator:
             if hasattr(obj, col_name):
                 # Convert all datetime values to strings.
                 setattr(obj, col_name, utils.datetime_to_str(col_val))
+
             if col_name == 'spec':
                 spec = col_val
 
         if spec:
-            obj = cls._set_input(obj, spec)
+            obj.set_attributes_from_spec(spec)
 
         return obj
 
 
+class CodeSource(resource.Resource, ScopedResource):
+    """CodeSource resource."""
+
+    id = wtypes.text
+    name = wtypes.text
+    namespace = wtypes.text
+    content = wtypes.text
+    version = wtypes.IntegerType(minimum=1)
+
+    project_id = wsme.wsattr(wtypes.text, readonly=True)
+    scope = SCOPE_TYPES
+
+    actions = [wtypes.text]
+
+    created_at = wtypes.text
+    updated_at = wtypes.text
+
+    @classmethod
+    def sample(cls):
+        return cls(
+            id='123e4567-e89b-12d3-a456-426655440000',
+            name='module',
+            content='content of file',
+            version=1,
+            scope='private',
+            actions=['action1', 'action2', 'action3'],
+            project_id='a7eb669e9819420ea4bd1453e672c0a7',
+            created_at='1970-01-01T00:00:00.000000',
+            updated_at='1970-01-01T00:00:00.000000',
+            namespace=''
+        )
+
+
+class CodeSources(resource.ResourceList):
+    """A collection of CodeSources."""
+
+    code_sources = [CodeSource]
+
+    def __init__(self, **kwargs):
+        self._type = 'code_sources'
+
+        super(CodeSources, self).__init__(**kwargs)
+
+    @classmethod
+    def sample(cls):
+        code_Source_sample = cls()
+        code_Source_sample.code_sources = [CodeSource.sample()]
+        code_Source_sample.next = (
+            "http://localhost:8989/v2/code_sources?"
+            "sort_keys=id,name&"
+            "sort_dirs=asc,desc&limit=10&"
+            "marker=123e4567-e89b-12d3-a456-426655440000"
+        )
+
+        return code_Source_sample
+
+
+class DynamicAction(resource.Resource, ScopedResource):
+    """DynamicAction resource."""
+
+    id = wtypes.text
+    name = wtypes.text
+    namespace = wsme.wsattr(wtypes.text, default='')
+    code_source_id = wtypes.text
+    code_source_name = wtypes.text
+    class_name = wtypes.text
+
+    project_id = wsme.wsattr(wtypes.text, readonly=True)
+    scope = SCOPE_TYPES
+
+    created_at = wtypes.text
+    updated_at = wtypes.text
+
+    @classmethod
+    def sample(cls):
+        return cls(
+            id='123e4567-e89b-12d3-a456-426655440000',
+            name='actionName',
+            class_name='className',
+            code_source_id='233e4567-354b-12d3-4444-426655444444',
+            code_source_name='my_sample_module',
+            scope='private',
+            project_id='a7eb669e9819420ea4bd1453e672c0a7',
+            created_at='1970-01-01T00:00:00.000000',
+            updated_at='1970-01-01T00:00:00.000000',
+            namespace=''
+        )
+
+
+class DynamicActions(resource.ResourceList):
+    """A collection of DynamicActions."""
+
+    dynamic_actions = [DynamicAction]
+
+    def __init__(self, **kwargs):
+        self._type = 'dynamic_actions'
+
+        super(DynamicActions, self).__init__(**kwargs)
+
+    @classmethod
+    def sample(cls):
+        dynamic_action_sample = cls()
+        dynamic_action_sample.dynamic_actions = [DynamicAction.sample()]
+        dynamic_action_sample.next = (
+            "http://localhost:8989/v2/dynamic_actions?"
+            "sort_keys=id,name&"
+            "sort_dirs=asc,desc&limit=10&"
+            "marker=123e4567-e89b-12d3-a456-426655440000"
+        )
+
+        return dynamic_action_sample
+
+
 class Workflows(resource.ResourceList):
     """A collection of workflows."""
 
@@ -205,6 +342,7 @@ class Action(resource.Resource, ScopedResource):
 
     created_at = wtypes.text
     updated_at = wtypes.text
+    namespace = wtypes.text
 
     @classmethod
     def sample(cls):
@@ -216,7 +354,8 @@ class Action(resource.Resource, ScopedResource):
             scope='private',
             project_id='a7eb669e9819420ea4bd1453e672c0a7',
             created_at='1970-01-01T00:00:00.000000',
-            updated_at='1970-01-01T00:00:00.000000'
+            updated_at='1970-01-01T00:00:00.000000',
+            namespace=''
         )
 
 
@@ -271,6 +410,9 @@ class Execution(resource.Resource):
     description = wtypes.text
     "description of workflow execution"
 
+    tags = [wtypes.text]
+    "tags of workflow execution"
+
     params = types.jsontype
     """'params' define workflow type specific parameters. Specific parameters
      are:
@@ -324,6 +466,7 @@ class Execution(resource.Resource):
             workflow_namespace='some_namespace',
             workflow_id='123e4567-e89b-12d3-a456-426655441111',
             description='this is the first execution.',
+            tags=['simple', 'amazing'],
             project_id='40a908dbddfe48ad80a87fb30fa70a03',
             state='SUCCESS',
             input={},
@@ -389,6 +532,8 @@ class Task(resource.Resource):
     workflow_id = wtypes.text
     workflow_execution_id = wtypes.text
 
+    tags = [wtypes.text]
+
     state = wtypes.text
     """state can take one of the following values:
      IDLE, RUNNING, SUCCESS, ERROR, DELAYED"""
@@ -412,7 +557,7 @@ class Task(resource.Resource):
     finished_at = wtypes.text
 
     # Add this param to make Mistral API work with WSME 0.8.0 or higher version
-    reset = wsme.wsattr(bool, mandatory=True)
+    reset = wsme.wsattr(bool)
 
     env = types.jsontype
 
@@ -423,6 +568,7 @@ class Task(resource.Resource):
             workflow_name='flow',
             workflow_id='123e4567-e89b-12d3-a456-426655441111',
             workflow_execution_id='123e4567-e89b-12d3-a456-426655440000',
+            tags=['long', 'security'],
             name='task',
             state=states.SUCCESS,
             project_id='40a908dbddfe48ad80a87fb30fa70a03',
@@ -823,6 +969,7 @@ class ExecutionReportStatistics(resource.Resource):
     error_tasks_count = wtypes.IntegerType(minimum=0)
     idle_tasks_count = wtypes.IntegerType(minimum=0)
     paused_tasks_count = wtypes.IntegerType(minimum=0)
+    estimated_time = wtypes.IntegerType(minimum=-1)
 
     def __init__(self, **kw):
         self.total_tasks_count = 0
@@ -831,6 +978,7 @@ class ExecutionReportStatistics(resource.Resource):
         self.error_tasks_count = 0
         self.idle_tasks_count = 0
         self.paused_tasks_count = 0
+        self.estimated_time = 0
 
         super(ExecutionReportStatistics, self).__init__(**kw)
 
@@ -862,7 +1010,8 @@ class ExecutionReportStatistics(resource.Resource):
             success_tasks_count=5,
             error_tasks_count=2,
             idle_tasks_count=0,
-            paused_tasks_count=0
+            paused_tasks_count=0,
+            estimated_time=40
         )
 
 
