@@ -29,6 +29,7 @@ from webtest import app as webtest_app
 
 from mistral.api.controllers.v2 import execution
 from mistral.api.controllers.v2 import resources
+from mistral import context
 from mistral.db.v2 import api as db_api
 from mistral.db.v2.sqlalchemy import api as sql_db_api
 from mistral.db.v2.sqlalchemy import models
@@ -169,6 +170,20 @@ class TestExecutionsController(base.APITest):
 
         self.assertDictEqual(expected, resp.json)
 
+    @mock.patch('mistral.db.v2.api.get_workflow_execution')
+    def test_get_with_fields_filter(self, mocked_get):
+        mocked_get.return_value = (
+            WF_EX_JSON_WITH_DESC['id'], WF_EX_JSON_WITH_DESC['description'],
+        )
+        resp = self.app.get('/v2/executions/123?fields=description')
+        expected = {
+            'id': WF_EX_JSON_WITH_DESC['id'],
+            'description': WF_EX_JSON_WITH_DESC['description'],
+        }
+
+        self.assertEqual(200, resp.status_int)
+        self.assertDictEqual(expected, resp.json)
+
     @mock.patch.object(db_api, 'get_workflow_execution')
     def test_get_operational_error(self, mocked_get):
         mocked_get.side_effect = [
@@ -214,7 +229,7 @@ class TestExecutionsController(base.APITest):
     @mock.patch.object(
         db_api,
         'get_workflow_execution',
-        mock.MagicMock(return_value=None)
+        mock.MagicMock(return_value=WF_EX)
     )
     @mock.patch.object(
         rpc_clients.EngineClient,
@@ -238,7 +253,7 @@ class TestExecutionsController(base.APITest):
     @mock.patch.object(
         db_api,
         'get_workflow_execution',
-        mock.MagicMock(return_value=None)
+        mock.MagicMock(return_value=WF_EX)
     )
     @mock.patch.object(rpc_clients.EngineClient, 'stop_workflow')
     def test_put_state_error(self, mock_stop_wf):
@@ -266,7 +281,7 @@ class TestExecutionsController(base.APITest):
     @mock.patch.object(
         db_api,
         'get_workflow_execution',
-        mock.MagicMock(return_value=None)
+        mock.MagicMock(return_value=WF_EX)
     )
     @mock.patch.object(rpc_clients.EngineClient, 'stop_workflow')
     def test_put_state_cancelled(self, mock_stop_wf):
@@ -299,7 +314,7 @@ class TestExecutionsController(base.APITest):
     @mock.patch.object(
         db_api,
         'get_workflow_execution',
-        mock.MagicMock(return_value=None)
+        mock.MagicMock(return_value=WF_EX)
     )
     @mock.patch.object(rpc_clients.EngineClient, 'resume_workflow')
     def test_put_state_resume(self, mock_resume_wf):
@@ -326,7 +341,7 @@ class TestExecutionsController(base.APITest):
     @mock.patch.object(
         db_api,
         'get_workflow_execution',
-        mock.MagicMock(return_value=None)
+        mock.MagicMock(return_value=WF_EX)
     )
     def test_put_invalid_state(self):
         invalid_states = [states.IDLE, states.WAITING, states.RUNNING_DELAYED]
@@ -353,7 +368,7 @@ class TestExecutionsController(base.APITest):
     @mock.patch.object(
         db_api,
         'get_workflow_execution',
-        mock.MagicMock(return_value=None)
+        mock.MagicMock(return_value=WF_EX)
     )
     @mock.patch.object(rpc_clients.EngineClient, 'stop_workflow')
     def test_put_state_info_unset(self, mock_stop_wf):
@@ -391,7 +406,8 @@ class TestExecutionsController(base.APITest):
 
         mock_ensure.assert_called_once_with(
             '123',
-            fields=(models.WorkflowExecution.id,)
+            fields=(models.WorkflowExecution.id,
+                    models.WorkflowExecution.root_execution_id)
         )
         mock_update.assert_called_once_with('123', update_params)
 
@@ -427,7 +443,7 @@ class TestExecutionsController(base.APITest):
     @mock.patch.object(
         db_api,
         'get_workflow_execution',
-        mock.MagicMock(return_value=None)
+        mock.MagicMock(return_value=WF_EX)
     )
     def test_put_empty(self):
         resp = self.app.put_json('/v2/executions/123', {}, expect_errors=True)
@@ -441,7 +457,7 @@ class TestExecutionsController(base.APITest):
     @mock.patch.object(
         db_api,
         'get_workflow_execution',
-        mock.MagicMock(return_value=None)
+        mock.MagicMock(return_value=WF_EX)
     )
     def test_put_state_and_description(self):
         resp = self.app.put_json(
@@ -487,7 +503,7 @@ class TestExecutionsController(base.APITest):
     @mock.patch.object(
         db_api,
         'get_workflow_execution',
-        mock.MagicMock(return_value=None)
+        mock.MagicMock(return_value=WF_EX)
     )
     def test_put_env_wrong_state(self):
         update_exec = {
@@ -564,7 +580,8 @@ class TestExecutionsController(base.APITest):
         self.assertEqual(201, resp.status_int)
         self.assertDictEqual(expected_json, resp.json)
 
-        load_wf_ex_func.assert_called_once_with(expected_json['id'])
+        load_wf_ex_func.assert_called_once_with(expected_json['id'],
+                                                fields=())
 
         kwargs = json.loads(expected_json['params'])
         kwargs['description'] = expected_json['description']
@@ -600,7 +617,8 @@ class TestExecutionsController(base.APITest):
         self.assertEqual(201, resp.status_int)
         self.assertDictEqual(expected_json, resp.json)
 
-        load_wf_ex_func.assert_called_once_with(expected_json['id'])
+        load_wf_ex_func.assert_called_once_with(expected_json['id'],
+                                                fields=())
 
         # Note that "start_workflow" method on engine API should not be called
         # in this case because we passed execution ID to the endpoint and the
@@ -877,7 +895,7 @@ class TestExecutionsController(base.APITest):
 
     def test_get_all_pagination_unknown_direction(self):
         resp = self.app.get(
-            '/v2/actions?limit=1&sort_keys=id&sort_dirs=nonexist',
+            '/v2/executions?limit=1&sort_keys=id&sort_dirs=nonexist',
             expect_errors=True
         )
 
@@ -994,3 +1012,37 @@ class TestExecutionsController(base.APITest):
             "The field 'id' can't hold None value.",
             resp.body.decode()
         )
+
+    @mock.patch.object(db_api, 'get_workflow_execution', MOCK_NOT_FOUND)
+    def test_get_all_tasks_within_execution(self):
+        fake_execution_id = "00000000-0000-0000-0000-000000000000"
+        resp = self.app.get(
+            '/v2/executions/%s/tasks' % fake_execution_id,
+            expect_errors=True
+        )
+
+        self.assertEqual(404, resp.status_int)
+        self.assertIn(
+            "Object not found",
+            resp.body.decode()
+        )
+
+    @mock.patch.object(rpc_clients.EngineClient, 'start_workflow')
+    def test_root_execution_id_present_in_logging_values(self, start_wf_func):
+        # NOTE: In fact, we use "white box" testing here to understand
+        # if the REST controller calls other APIs as expected. This is
+        # the only way of testing available with the current testing
+        # infrastructure.
+        wf_ex_dict = WF_EX.to_dict()
+        start_wf_func.return_value = wf_ex_dict
+        json_body = WF_EX_JSON_WITH_DESC.copy()
+        exp_root_execution_id = WF_EX_JSON["id"]
+
+        with mock.patch("mistral.context.set_ctx") as mocked_set_cxt:
+            self.app.post_json('/v2/executions', json_body)
+            calls = mocked_set_cxt.call_args_list
+            ctx = calls[0][0][0]
+            self.assertIsInstance(ctx, context.MistralContext)
+            logging_values = ctx.get_logging_values()
+            self.assertEqual(exp_root_execution_id,
+                             logging_values["root_execution_id"])
