@@ -1,5 +1,6 @@
 # Copyright 2013 - Mirantis, Inc.
 # Copyright 2015 - StackStorm, Inc.
+# Copyright 2019 - NetCracker Technology Corp.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License");
 #    you may not use this file except in compliance with the License.
@@ -16,8 +17,8 @@
 import copy
 import datetime
 import json
+from unittest import mock
 
-import mock
 import sqlalchemy as sa
 
 from mistral.db.v2 import api as db_api
@@ -109,6 +110,7 @@ TASK = {
     'name': 'task',
     'workflow_name': 'flow',
     'workflow_id': '123e4567-e89b-12d3-a456-426655441111',
+    'tags': ['a', 'b'],
     'state': 'RUNNING',
     'workflow_execution_id': WF_EX.id,
     'created_at': '1970-01-01 00:00:00',
@@ -136,6 +138,11 @@ ERROR_ITEMS_TASK_EX['state'] = 'ERROR'
 ERROR_TASK = copy.deepcopy(TASK)
 ERROR_TASK['state'] = 'ERROR'
 
+SKIPPED_TASK_EX = copy.deepcopy(TASK_EX)
+SKIPPED_TASK_EX['state'] = 'SKIPPED'
+SKIPPED_TASK = copy.deepcopy(TASK)
+SKIPPED_TASK['state'] = 'SKIPPED'
+
 BROKEN_TASK = copy.deepcopy(TASK)
 
 RERUN_TASK = {
@@ -143,6 +150,11 @@ RERUN_TASK = {
     'state': 'RUNNING'
 }
 
+SKIP_TASK = {
+    'id': '123',
+    'state': 'SKIPPED'
+}
+
 MOCK_WF_EX = mock.MagicMock(return_value=WF_EX)
 TASK_EX.workflow_execution = WF_EX
 MOCK_TASK = mock.MagicMock(return_value=TASK_EX)
@@ -169,6 +181,18 @@ class TestTasksController(base.APITest):
         self.assertEqual(200, resp.status_int)
         self.assertDictEqual(TASK, resp.json)
 
+    @mock.patch('mistral.db.v2.api.get_task_execution')
+    def test_get_with_fields_filter(self, mocked_get):
+        mocked_get.return_value = TASK_EX
+        resp = self.app.get('/v2/tasks/123?fields=name')
+        expected = {
+            'id': TASK['id'],
+            'name': TASK['name'],
+        }
+
+        self.assertEqual(200, resp.status_int)
+        self.assertDictEqual(expected, resp.json)
+
     @mock.patch.object(db_api, 'get_task_execution')
     def test_get_operational_error(self, mocked_get):
         mocked_get.side_effect = [
@@ -235,7 +259,7 @@ class TestTasksController(base.APITest):
         mock.MagicMock(side_effect=[ERROR_TASK_EX, TASK_EX])
     )
     @mock.patch.object(rpc.EngineClient, 'rerun_workflow', MOCK_WF_EX)
-    def test_put(self):
+    def test_put_rerun(self):
         params = copy.deepcopy(RERUN_TASK)
         params['reset'] = True
 
@@ -247,6 +271,7 @@ class TestTasksController(base.APITest):
         rpc.EngineClient.rerun_workflow.assert_called_with(
             TASK_EX.id,
             reset=params['reset'],
+            skip=False,
             env=None
         )
 
@@ -257,7 +282,29 @@ class TestTasksController(base.APITest):
         mock.MagicMock(side_effect=[ERROR_TASK_EX, TASK_EX])
     )
     @mock.patch.object(rpc.EngineClient, 'rerun_workflow', MOCK_WF_EX)
-    def test_put_missing_reset(self):
+    def test_put_skip(self):
+        params = copy.deepcopy(SKIP_TASK)
+
+        resp = self.app.put_json('/v2/tasks/123', params=params)
+
+        self.assertEqual(200, resp.status_int)
+        self.assertDictEqual(TASK, resp.json)
+
+        rpc.EngineClient.rerun_workflow.assert_called_with(
+            TASK_EX.id,
+            reset=None,
+            skip=True,
+            env=None
+        )
+
+    @mock.patch.object(db_api, 'get_workflow_execution', MOCK_WF_EX)
+    @mock.patch.object(
+        db_api,
+        'get_task_execution',
+        mock.MagicMock(side_effect=[ERROR_TASK_EX, TASK_EX])
+    )
+    @mock.patch.object(rpc.EngineClient, 'rerun_workflow', MOCK_WF_EX)
+    def test_put_missing_reset_rerun(self):
         params = copy.deepcopy(RERUN_TASK)
 
         resp = self.app.put_json(
@@ -267,7 +314,10 @@ class TestTasksController(base.APITest):
 
         self.assertEqual(400, resp.status_int)
         self.assertIn('faultstring', resp.json)
-        self.assertIn('Mandatory field missing', resp.json['faultstring'])
+        self.assertIn(
+            'Reset field is mandatory to rerun task',
+            resp.json['faultstring']
+        )
 
     @mock.patch.object(db_api, 'get_workflow_execution', MOCK_WF_EX)
     @mock.patch.object(
@@ -276,7 +326,7 @@ class TestTasksController(base.APITest):
         mock.MagicMock(side_effect=[ERROR_ITEMS_TASK_EX, WITH_ITEMS_TASK_EX])
     )
     @mock.patch.object(rpc.EngineClient, 'rerun_workflow', MOCK_WF_EX)
-    def test_put_with_items(self):
+    def test_put_with_items_rerun(self):
         params = copy.deepcopy(RERUN_TASK)
         params['reset'] = False
 
@@ -305,12 +355,13 @@ class TestTasksController(base.APITest):
         rpc.EngineClient.rerun_workflow.assert_called_with(
             TASK_EX.id,
             reset=params['reset'],
+            skip=False,
             env=json.loads(params['env'])
         )
 
     @mock.patch.object(db_api, 'get_workflow_execution', MOCK_WF_EX)
     @mock.patch.object(db_api, 'get_task_execution', MOCK_TASK)
-    def test_put_current_task_not_in_error(self):
+    def test_put_current_task_not_in_error_rerun(self):
         params = copy.deepcopy(RERUN_TASK)
         params['reset'] = True
 
@@ -324,10 +375,25 @@ class TestTasksController(base.APITest):
         self.assertIn('faultstring', resp.json)
         self.assertIn('execution must be in ERROR', resp.json['faultstring'])
 
+    @mock.patch.object(db_api, 'get_workflow_execution', MOCK_WF_EX)
+    @mock.patch.object(db_api, 'get_task_execution', MOCK_TASK)
+    def test_put_current_task_not_in_error_skip(self):
+        params = copy.deepcopy(SKIP_TASK)
+
+        resp = self.app.put_json(
+            '/v2/tasks/123',
+            params=params,
+            expect_errors=True
+        )
+
+        self.assertEqual(400, resp.status_int)
+        self.assertIn('faultstring', resp.json)
+        self.assertIn('execution must be in ERROR', resp.json['faultstring'])
+
     @mock.patch.object(rpc.EngineClient, 'rerun_workflow', MOCK_WF_EX)
     @mock.patch.object(db_api, 'get_workflow_execution', MOCK_WF_EX)
     @mock.patch.object(db_api, 'get_task_execution', MOCK_ERROR_TASK)
-    def test_put_current_task_in_error(self):
+    def test_put_current_task_in_error_rerun(self):
         params = copy.deepcopy(RERUN_TASK)
         params['reset'] = True
         params['env'] = '{"k1": "def"}'
@@ -338,7 +404,7 @@ class TestTasksController(base.APITest):
 
     @mock.patch.object(db_api, 'get_workflow_execution', MOCK_WF_EX)
     @mock.patch.object(db_api, 'get_task_execution', MOCK_ERROR_TASK)
-    def test_put_invalid_state(self):
+    def test_put_invalid_state_rerun(self):
         params = copy.deepcopy(RERUN_TASK)
         params['state'] = states.IDLE
         params['reset'] = True
@@ -355,7 +421,7 @@ class TestTasksController(base.APITest):
 
     @mock.patch.object(db_api, 'get_workflow_execution', MOCK_WF_EX)
     @mock.patch.object(db_api, 'get_task_execution', MOCK_ERROR_TASK)
-    def test_put_invalid_reset(self):
+    def test_put_invalid_reset_rerun(self):
         params = copy.deepcopy(RERUN_TASK)
         params['reset'] = False
 
@@ -369,23 +435,24 @@ class TestTasksController(base.APITest):
         self.assertIn('faultstring', resp.json)
         self.assertIn('Only with-items task', resp.json['faultstring'])
 
-        @mock.patch.object(db_api, 'get_workflow_execution', MOCK_WF_EX)
-        @mock.patch.object(db_api, 'get_task_execution', MOCK_ERROR_TASK)
-        def test_put_valid_state(self):
-            params = copy.deepcopy(RERUN_TASK)
-            params['state'] = states.RUNNING
-            params['reset'] = True
+    @mock.patch.object(rpc.EngineClient, 'rerun_workflow', MOCK_WF_EX)
+    @mock.patch.object(db_api, 'get_workflow_execution', MOCK_WF_EX)
+    @mock.patch.object(db_api, 'get_task_execution', MOCK_ERROR_TASK)
+    def test_put_valid_state_rerun(self):
+        params = copy.deepcopy(RERUN_TASK)
+        params['state'] = states.RUNNING
+        params['reset'] = True
 
-            resp = self.app.put_json(
-                '/v2/tasks/123',
-                params=params
-            )
+        resp = self.app.put_json(
+            '/v2/tasks/123',
+            params=params
+        )
 
-            self.assertEqual(200, resp.status_int)
+        self.assertEqual(200, resp.status_int)
 
     @mock.patch.object(db_api, 'get_workflow_execution', MOCK_WF_EX)
     @mock.patch.object(db_api, 'get_task_execution', MOCK_ERROR_TASK)
-    def test_put_mismatch_task_name(self):
+    def test_put_mismatch_task_name_rerun(self):
         params = copy.deepcopy(RERUN_TASK)
         params['name'] = 'abc'
         params['reset'] = True
@@ -403,7 +470,7 @@ class TestTasksController(base.APITest):
     @mock.patch.object(rpc.EngineClient, 'rerun_workflow', MOCK_WF_EX)
     @mock.patch.object(db_api, 'get_workflow_execution', MOCK_WF_EX)
     @mock.patch.object(db_api, 'get_task_execution', MOCK_ERROR_TASK)
-    def test_put_match_task_name(self):
+    def test_put_match_task_name_rerun(self):
         params = copy.deepcopy(RERUN_TASK)
         params['name'] = 'task'
         params['reset'] = True
@@ -418,7 +485,7 @@ class TestTasksController(base.APITest):
 
     @mock.patch.object(db_api, 'get_workflow_execution', MOCK_WF_EX)
     @mock.patch.object(db_api, 'get_task_execution', MOCK_ERROR_TASK)
-    def test_put_mismatch_workflow_name(self):
+    def test_put_mismatch_workflow_name_rerun(self):
         params = copy.deepcopy(RERUN_TASK)
         params['workflow_name'] = 'xyz'
         params['reset'] = True
