@@ -15,16 +15,19 @@
 
 import copy
 import datetime
+from unittest import mock
 
-import mock
+import json
 import sqlalchemy as sa
+import yaml
 
 from mistral.db.v2 import api as db_api
 from mistral.db.v2.sqlalchemy import models
 from mistral import exceptions as exc
 from mistral.tests.unit.api import base
 from mistral.tests.unit import base as unit_base
-from mistral import utils
+from mistral.utils import safe_yaml
+from mistral_lib import utils
 
 WF_DEFINITION = """
 ---
@@ -58,7 +61,8 @@ WF = {
     'definition': WF_DEFINITION,
     'created_at': '1970-01-01 00:00:00',
     'updated_at': '1970-01-01 00:00:00',
-    'input': 'param1'
+    'input': 'param1',
+    'interface': {"output": [], "input": ["param1"]}
 }
 
 WF_DB_WITHIN_ABC_NAMESPACE = models.WorkflowDefinition(
@@ -78,7 +82,8 @@ WF_WITH_NAMESPACE = {
     'definition': WF_DEFINITION,
     'created_at': '1970-01-01 00:00:00',
     'updated_at': '1970-01-01 00:00:00',
-    'input': 'param1'
+    'input': 'param1',
+    'interface': {'input': ['param1'], 'output': []}
 }
 
 WF_DEFINITION_WITH_INPUT = """
@@ -109,7 +114,11 @@ WF_WITH_DEFAULT_INPUT = {
     'definition': WF_DEFINITION_WITH_INPUT,
     'created_at': '1970-01-01 00:00:00',
     'updated_at': '1970-01-01 00:00:00',
-    'input': 'param1, param2=2'
+    'input': 'param1, param2="2"',
+    'interface': {
+        "input": ["param1", {"param2": 2}],
+        "output": []
+    }
 }
 
 WF_DB_PROJECT_ID = WF_DB.get_clone()
@@ -179,6 +188,60 @@ wf2:
       action: std.echo output="Mistral"
 """
 
+WFS_YAML = safe_yaml.safe_load(WFS_DEFINITION)
+FIRST_WF_DEF = yaml.dump({
+    'version': '2.0',
+    'wf1': WFS_YAML['wf1']
+})
+SECOND_WF_DEF = yaml.dump({
+    'version': '2.0',
+    'wf2': WFS_YAML['wf2']
+})
+
+FIRST_WF_DICT = {
+    'name': 'wf1',
+    'tasks': {
+        'task1': {
+            'action': 'std.echo output="Hello"',
+            'name': 'task1',
+            'type': 'direct',
+            'version': '2.0'
+        }
+    },
+    'version': '2.0'
+}
+FIRST_WF = {
+    'name': 'wf1',
+    'tags': [],
+    'definition': FIRST_WF_DEF,
+    'spec': FIRST_WF_DICT,
+    'scope': 'private',
+    'namespace': '',
+    'is_system': False
+}
+
+SECOND_WF_DICT = {
+    'name': 'wf2',
+    'tasks': {
+        'task1': {
+            'action': 'std.echo output="Mistral"',
+            'name': 'task1',
+            'type': 'direct',
+            'version': '2.0'
+        }
+    },
+    'version': '2.0'
+}
+SECOND_WF = {
+    'name': 'wf2',
+    'tags': [],
+    'definition': SECOND_WF_DEF,
+    'spec': SECOND_WF_DICT,
+    'scope': 'private',
+    'namespace': '',
+    'is_system': False
+}
+
 MOCK_WF = mock.MagicMock(return_value=WF_DB)
 MOCK_WF_SYSTEM = mock.MagicMock(return_value=WF_DB_SYSTEM)
 MOCK_WF_WITH_INPUT = mock.MagicMock(return_value=WF_DB_WITH_INPUT)
@@ -195,8 +258,23 @@ class TestWorkflowsController(base.APITest):
     def test_get(self):
         resp = self.app.get('/v2/workflows/123')
 
+        resp_json = resp.json
+        resp_json['interface'] = json.loads(resp_json['interface'])
+
         self.assertEqual(200, resp.status_int)
-        self.assertDictEqual(WF, resp.json)
+        self.assertDictEqual(WF, resp_json)
+
+    @mock.patch('mistral.db.v2.api.get_workflow_definition')
+    def test_get_with_fields_filter(self, mocked_get):
+        mocked_get.return_value = (WF['id'], WF['name'],)
+        resp = self.app.get('/v2/workflows/123?fields=name')
+        expected = {
+            'id': WF['id'],
+            'name': WF['name'],
+        }
+
+        self.assertEqual(200, resp.status_int)
+        self.assertDictEqual(expected, resp.json)
 
     @mock.patch.object(db_api, 'get_workflow_definition')
     def test_get_operational_error(self, mocked_get):
@@ -208,8 +286,11 @@ class TestWorkflowsController(base.APITest):
 
         resp = self.app.get('/v2/workflows/123')
 
+        resp_json = resp.json
+        resp_json['interface'] = json.loads(resp_json['interface'])
+
         self.assertEqual(200, resp.status_int)
-        self.assertDictEqual(WF, resp.json)
+        self.assertDictEqual(WF, resp_json)
 
     @mock.patch.object(db_api, "get_workflow_definition", MOCK_WF_WITH_INPUT)
     def test_get_with_input(self):
@@ -217,8 +298,11 @@ class TestWorkflowsController(base.APITest):
 
         self.maxDiff = None
 
+        resp_json = resp.json
+        resp_json['interface'] = json.loads(resp_json['interface'])
+
         self.assertEqual(200, resp.status_int)
-        self.assertDictEqual(WF_WITH_DEFAULT_INPUT, resp.json)
+        self.assertDictEqual(WF_WITH_DEFAULT_INPUT, resp_json)
 
     @mock.patch.object(db_api, "get_workflow_definition", MOCK_NOT_FOUND)
     def test_get_not_found(self):
@@ -256,7 +340,8 @@ class TestWorkflowsController(base.APITest):
             UPDATED_WF_DEFINITION,
             scope='private',
             identifier='123e4567-e89b-12d3-a456-426655440000',
-            namespace=''
+            namespace='',
+            validate=True
         )
         self.assertDictEqual(UPDATED_WF, resp.json)
 
@@ -293,6 +378,17 @@ class TestWorkflowsController(base.APITest):
 
         self.assertEqual("public", mock_update.call_args[0][1]['scope'])
 
+    def test_put_wrong_scope(self):
+        resp = self.app.put(
+            '/v2/workflows?scope=unique',
+            UPDATED_WF_DEFINITION,
+            headers={'Content-Type': 'text/plain'},
+            expect_errors=True
+        )
+
+        self.assertEqual(400, resp.status_int)
+        self.assertIn("Scope must be one of the following", resp.body.decode())
+
     @mock.patch.object(
         db_api, "update_workflow_definition", MOCK_WF_WITH_INPUT
     )
@@ -332,6 +428,36 @@ class TestWorkflowsController(base.APITest):
         self.assertEqual(400, resp.status_int)
         self.assertIn("Invalid DSL", resp.body.decode())
 
+    @mock.patch.object(
+        db_api, "update_workflow_definition", MOCK_UPDATED_WF
+    )
+    def test_put_invalid_skip_validation(self):
+        self.override_config('validation_mode', 'enabled', 'api')
+
+        resp = self.app.put(
+            '/v2/workflows?skip_validation',
+            WF_DEF_INVALID_MODEL_EXCEPTION,
+            headers={'Content-Type': 'text/plain'},
+            expect_errors=True
+        )
+
+        self.assertEqual(200, resp.status_int)
+
+    @mock.patch.object(db_api, "update_workflow_definition")
+    def test_put_multiple(self, mock_mtd):
+        spec_mock = mock_mtd.return_value.get.return_value
+        spec_mock.get.return_value = {}
+
+        self.app.put(
+            '/v2/workflows',
+            WFS_DEFINITION,
+            headers={'Content-Type': 'text/plain'}
+        )
+
+        self.assertEqual(2, mock_mtd.call_count)
+        mock_mtd.assert_any_call('wf1', FIRST_WF)
+        mock_mtd.assert_any_call('wf2', SECOND_WF)
+
     def test_put_more_workflows_with_uuid(self):
         resp = self.app.put(
             '/v2/workflows/123e4567-e89b-12d3-a456-426655440000',
@@ -381,10 +507,7 @@ class TestWorkflowsController(base.APITest):
 
         self.assertEqual("public", mock_mtd.call_args[0][0]['scope'])
 
-    @mock.patch.object(db_api, "create_action_definition")
-    def test_post_wrong_scope(self, mock_mtd):
-        mock_mtd.return_value = WF_DB
-
+    def test_post_wrong_scope(self):
         resp = self.app.post(
             '/v2/workflows?scope=unique',
             WF_DEFINITION,
@@ -406,6 +529,21 @@ class TestWorkflowsController(base.APITest):
 
         self.assertEqual(409, resp.status_int)
 
+    @mock.patch.object(db_api, "create_workflow_definition")
+    def test_post_multiple(self, mock_mtd):
+        spec_mock = mock_mtd.return_value.get.return_value
+        spec_mock.get.return_value = {}
+
+        self.app.post(
+            '/v2/workflows',
+            WFS_DEFINITION,
+            headers={'Content-Type': 'text/plain'}
+        )
+
+        self.assertEqual(2, mock_mtd.call_count)
+        mock_mtd.assert_any_call(FIRST_WF)
+        mock_mtd.assert_any_call(SECOND_WF)
+
     def test_post_invalid(self):
         resp = self.app.post(
             '/v2/workflows',
@@ -417,6 +555,18 @@ class TestWorkflowsController(base.APITest):
         self.assertEqual(400, resp.status_int)
         self.assertIn("Invalid DSL", resp.body.decode())
 
+    def test_post_invalid_skip_validation(self):
+        self.override_config('validation_mode', 'enabled', 'api')
+
+        resp = self.app.post(
+            '/v2/workflows?skip_validation',
+            WF_DEF_INVALID_MODEL_EXCEPTION,
+            headers={'Content-Type': 'text/plain'},
+            expect_errors=True
+        )
+
+        self.assertEqual(201, resp.status_int)
+
     @mock.patch.object(db_api, "delete_workflow_definition", MOCK_DELETE)
     @mock.patch.object(db_api, "get_workflow_definition", MOCK_WF)
     def test_delete(self):
@@ -449,8 +599,11 @@ class TestWorkflowsController(base.APITest):
 
         self.assertEqual(200, resp.status_int)
 
+        resp_json = resp.json['workflows'][0]
+        resp_json['interface'] = json.loads(resp_json['interface'])
+
         self.assertEqual(1, len(resp.json['workflows']))
-        self.assertDictEqual(WF, resp.json['workflows'][0])
+        self.assertDictEqual(WF, resp_json)
 
     @mock.patch.object(db_api, 'get_workflow_definitions')
     def test_get_all_operational_error(self, mocked_get_all):
@@ -462,10 +615,13 @@ class TestWorkflowsController(base.APITest):
 
         resp = self.app.get('/v2/workflows')
 
-        self.assertEqual(200, resp.status_int)
+        resp_workflow_json = resp.json['workflows'][0]
+        resp_workflow_json['interface'] = \
+            json.loads(resp_workflow_json['interface'])
 
+        self.assertEqual(200, resp.status_int)
         self.assertEqual(1, len(resp.json['workflows']))
-        self.assertDictEqual(WF, resp.json['workflows'][0])
+        self.assertDictEqual(WF, resp_workflow_json)
 
     @mock.patch.object(db_api, "get_workflow_definitions", MOCK_EMPTY)
     def test_get_all_empty(self):
@@ -504,8 +660,12 @@ class TestWorkflowsController(base.APITest):
 
         self.assertIn('next', resp.json)
 
+        resp_workflow_json = resp.json['workflows'][0]
+        resp_workflow_json['interface'] = \
+            json.loads(resp_workflow_json['interface'])
+
         self.assertEqual(1, len(resp.json['workflows']))
-        self.assertDictEqual(WF, resp.json['workflows'][0])
+        self.assertDictEqual(WF, resp_workflow_json)
 
         param_dict = utils.get_dict_from_string(
             resp.json['next'].split('?')[1],
@@ -582,6 +742,42 @@ class TestWorkflowsController(base.APITest):
 
         self.assertDictEqual(expected_dict, resp.json['workflows'][0])
 
+    @mock.patch('mistral.db.v2.api.get_workflow_definitions')
+    def test_get_all_with_fields_input_filter(self, mock_get_db_wfs):
+        expected_dict = {
+            'id': '65df1f59-938f-4c17-bc2a-562524ef5e40',
+            'input': 'param1, param2="2"',
+            'interface': {
+                "output": [],
+                "input": ["param1",
+                          {"param2": 2}
+                          ]
+            }
+        }
+
+        def mock_get_defintions(fields=None, session=None, **kwargs):
+            if fields and 'input' in fields:
+                fields.remove('input')
+                fields.append('spec')
+
+            return [
+                ('65df1f59-938f-4c17-bc2a-562524ef5e40',
+                 {'input': ['param1', {'param2': 2}]})
+            ]
+
+        mock_get_db_wfs.side_effect = mock_get_defintions
+
+        resp = self.app.get('/v2/workflows?fields=input')
+
+        self.assertEqual(200, resp.status_int)
+        self.assertEqual(1, len(resp.json['workflows']))
+
+        resp_workflow_json = resp.json['workflows'][0]
+        resp_workflow_json['interface'] = \
+            json.loads(resp_workflow_json['interface'])
+
+        self.assertDictEqual(expected_dict, resp_workflow_json)
+
     def test_get_all_with_invalid_field(self):
         resp = self.app.get(
             '/v2/workflows?fields=name,nonexist',
@@ -590,10 +786,9 @@ class TestWorkflowsController(base.APITest):
 
         self.assertEqual(400, resp.status_int)
 
-        self.assertIn(
-            "nonexist are invalid",
-            resp.body.decode()
-        )
+        response_msg = resp.body.decode()
+        self.assertIn("nonexist", response_msg)
+        self.assertIn("do not exist", response_msg)
 
     def test_validate(self):
         resp = self.app.post(
@@ -692,7 +887,8 @@ class TestWorkflowsController(base.APITest):
             WF_DEFINITION,
             scope='private',
             identifier=id_,
-            namespace='abc'
+            namespace='abc',
+            validate=True
         )
         self.assertDictEqual(WF_WITH_NAMESPACE, resp.json)
 
@@ -702,4 +898,4 @@ class TestWorkflowsController(base.APITest):
         resp = self.app.get(
             '/v2/workflows/123e4567-e89b-12d3-a456-426655440000')
         self.assertEqual(200, resp.status_int)
-        self.assertTrue('project_id' in resp.json)
+        self.assertIn('project_id', resp.json)
